/*-
 * Copyright (c) 2009,2010 Kai Wang
 * Copyright (c) 2007,2008 Hyogeol Lee <hyogeollee@gmail.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer
 *    in this position and unchanged.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef	__LIBELFTC_H_
#define	__LIBELFTC_H_

#include <stdbool.h>

#include "_elftc.h"

struct _Ar_Obj {
	void		*ao_handle;	/* archive obj handle. */
	Ar_Backend	 ao_be;		/* archive backend. */
	Ar_Cmd	 	 ao_cmd;	/* archive mode. */
	Ar_Compress	 ao_cmp;	/* compression mode. */
	Ar_Flavor	 ao_flv;	/* archive flavor. */
	Elf_Arsym	*ao_symtab;	/* archive symbol table. */
	char		*ao_strtab;	/* archive string table. */
	size_t		 ao_strtabsz;	/* archive strint table size. */
	off_t		 ao_off;	/* current member offset. */
	int		 ao_flags;	/* archive flags. */
	struct _Ar_Member *ao_pos;	/* current member pointer. */
	TAILQ_HEAD(am_head, _Ar_Member) ao_amlist; /* member list. */
};

struct _Ar_Member {
	Elf		*am_elf;	/* archive member ELF descr. */
	Elf_Arhdr	*am_hdr;	/* archive member header. */
	off_t		 am_off;	/* archive member offset. */
	void		*am_data;	/* archive member content. */
	TAILQ_ENTRY(_Ar_Member) am_next;/* next archive member. */
};

struct _Ar_Backend_CB {
	Ar_Obj *(*open_fd)(int, Ar_Cmd, int);
	Ar_Obj *(*open_file)(const char *, Ar_Cmd, int);
	Ar_Obj *(*open_memory)(void *, size_t, int);
	int (*set_flavor)(Ar_Obj *, Ar_Flavor);
	int (*set_compression)(Ar_Obj *, Ar_Compress);
	int (*next)(Ar_Obj *);
	int (*rand)(Ar_Obj *, off_t);
	int (*rewind)(Ar_Obj *);
	Elf_Arsym *(*get_arsym)(Ar_Obj *, size_t *);
	Elf_Arhdr *(*get_arhdr)(Ar_Obj *);
	void *(*get_data)(Ar_Obj *, size_t *);
	Elf *(*get_elf)(Ar_Obj *);
	int (*extract)(Ar_Obj *, int flags);
	int (*new_arhdr)(Ar_Obj *, Elf_Arhdr *);
	int (*new_data)(Ar_Obj *, void *, size_t);
	int (*close)(Ar_Obj *);
};

struct _Bfd_Target {
	const char	*bt_name;	/* target name. */
	unsigned int	 bt_type;	/* target type. */
	unsigned int	 bt_byteorder;	/* elf target byteorder. */
	unsigned int	 bt_elfclass;	/* elf target class (32/64bit). */
	unsigned int	 bt_machine;	/* elf target arch. */
	unsigned int	 bt_osabi;	/* elf target abi. */
};

extern struct _Bfd_Target _libelftc_targets[];

/** @brief Dynamic vector data for string. */
struct vector_str {
	/** Current size */
	size_t		size;
	/** Total capacity */
	size_t		capacity;
	/** String array */
	char		**container;
};

#define LIBELFTC_MSG_SIZE	1024

struct _libelftc_globals {
	int libelftc_error;
	char libelftc_msg[LIBELFTC_MSG_SIZE];
};

extern struct _libelftc_globals _libelftc;
extern const char *_libelftc_errors[];

#define	LIBELFTC_PRIVATE(N)	(_libelftc.libelftc_##N)

#define	LIBELFTC_ELFTC_ERROR_MASK	0xFF
#define	LIBELFTC_OS_ERROR_SHIFT		8

#define	ELFTC_SET_ERROR(E, O) do {					\
	LIBELFTC_PRIVATE(error) =					\
	    ((ELFTC_E_##E & LIBELFTC_ELFTC_ERROR_MASK) |		\
	    ((O) << LIBELFTC_OS_ERROR_SHIFT));				\
	_elftc_gen_errmsg(LIBELFTC_PRIVATE(error), NULL, NULL);		\
	} while (0)

#define	ELFTC_SET_ERROR_MSG(E, M1, M2) do {				\
	LIBELFTC_PRIVATE(error) = (ELFTC_E_##E);			\
	_elftc_gen_errmsg(LIBELFTC_PRIVATE(error), (M1), (M2));		\
	} while (0)

#define	ELFTC_SET_ELF_ERROR(M)						\
	ELFTC_SET_ERROR_MSG(LIBELF, (M), elf_errmsg(elf_errno()))

#define	ELFTC_SET_ARCHIVE_ERROR(M)					\
	ELFTC_SET_ERROR_MSG(LIBARCHIVE, (M), archive_error_string(a))

#define BUFFER_GROWFACTOR	1.618
#define VECTOR_DEF_CAPACITY	8

Ar_Obj	*_elftc_ar_alloc_object(Ar_Cmd, Ar_Backend, int);
void	_elftc_ar_dealloc_object(Ar_Obj *);
int	_elftc_ar_libarchive_close(Ar_Obj *);
Elf_Arhdr *_elftc_ar_libarchive_get_arhdr(Ar_Obj *);
void	*_elftc_ar_libarchive_get_data(Ar_Obj *, size_t *);
Elf	*_elftc_ar_libarchive_get_elf(Ar_Obj *);
int	_elftc_ar_libarchive_next(Ar_Obj *);
Ar_Obj	*_elftc_ar_libarchive_open_fd(int, Ar_Cmd, int);
Ar_Obj	*_elftc_ar_libarchive_open_file(const char *, Ar_Cmd, int);
Ar_Obj	*_elftc_ar_libarchive_open_memory(void *, size_t, int);
int	_elftc_ar_libarchive_rand(Ar_Obj *, off_t);
int	_elftc_ar_libarchive_rewind(Ar_Obj *);
int	_elftc_ar_libelf_next(Ar_Obj *);
Elf_Arhdr *_elftc_ar_libelf_get_arhdr(Ar_Obj *);
int	_elftc_ar_libelf_close(Ar_Obj *);
Elf_Arsym *_elftc_ar_libelf_get_arsym(Ar_Obj *, size_t *);
void	*_elftc_ar_libelf_get_data(Ar_Obj *, size_t *);
Elf	*_elftc_ar_libelf_get_elf(Ar_Obj *);
Ar_Obj	*_elftc_ar_libelf_open_fd(int, Ar_Cmd, int);
Ar_Obj	*_elftc_ar_libelf_open_file(const char *, Ar_Cmd, int);
Ar_Obj	*_elftc_ar_libelf_open_memory(void *, size_t, int);
int	_elftc_ar_libelf_rand(Ar_Obj *, off_t);
int	_elftc_ar_libelf_rewind(Ar_Obj *);
void	_elftc_gen_errmsg(int, const char *, const char *);
void	vector_str_dest(struct vector_str *);
int	vector_str_find(const struct vector_str *, const char *, size_t);
char	*vector_str_get_flat(const struct vector_str *, size_t *);
bool	vector_str_init(struct vector_str *);
bool	vector_str_pop(struct vector_str *);
bool	vector_str_push(struct vector_str *, const char *, size_t);
bool	vector_str_push_vector_head(struct vector_str *, struct vector_str *);
char	*vector_str_substr(const struct vector_str *, size_t, size_t, size_t *);
char	*cpp_demangle_gnu3(const char *);
bool	is_cpp_mangled_gnu3(const char *);
char	*cpp_demangle_gnu2(const char *);
bool	is_cpp_mangled_gnu2(const char *);
char	*cpp_demangle_ARM(const char *);
bool	is_cpp_mangled_ARM(const char *);

/*
 * Symbols that are sometimes missing from system headers.
 *
 * TODO: Move these to a project-wide file.
 */

#ifndef	EM_AVR
#define	EM_AVR		83
#endif

#endif	/* __LIBELFTC_H */
