/*-
 * Copyright (c) 2010 Kai Wang
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <libelftc.h>
#include <stdio.h>
#include <string.h>

#include "_libelftc.h"

struct _libelftc_globals _libelftc = {
	.libelftc_error = 0,
	.libelftc_msg	= "",
};

const char *_libelftc_errors[] = {
#define	DEFINE_ERROR(N,S)	[ELFTC_E_##N] = S
	DEFINE_ERROR(NONE,		"No error"),
	DEFINE_ERROR(ARGUMENT,		"Invalid argument"),
	DEFINE_ERROR(IO,		"I/O error"),
	DEFINE_ERROR(LIBARCHIVE,	"libarchive error"),
	DEFINE_ERROR(LIBELF,		"libelf error"),
	DEFINE_ERROR(RESOURCE,		"Resource exhaustion"),	
	DEFINE_ERROR(SEQUENCE,		"API calls out of sequence"),
	DEFINE_ERROR(UNIMPL,		"Unimplemented feature"),
	DEFINE_ERROR(NUM,		"Unknown error")
#undef	DEFINE_ERROR
};

void
_elftc_gen_errmsg(int error, const char *msg1, const char *msg2)
{
	int oserr;

	oserr = error >> LIBELFTC_OS_ERROR_SHIFT;
	error &= LIBELFTC_ELFTC_ERROR_MASK;

	if (msg1 != NULL) {
		if (msg2 != NULL)
			snprintf(LIBELFTC_PRIVATE(msg),
			    sizeof(LIBELFTC_PRIVATE(msg)),
			    "%s: %s: %s", _libelftc_errors[error], msg1, msg2);
		else
			snprintf(LIBELFTC_PRIVATE(msg),
			    sizeof(LIBELFTC_PRIVATE(msg)),
			    "%s: %s", _libelftc_errors[error], msg1);
		return;
	}

	if (oserr)
		snprintf(LIBELFTC_PRIVATE(msg), sizeof(LIBELFTC_PRIVATE(msg)),
		    "%s: %s", _libelftc_errors[error], strerror(oserr));
	else
		snprintf(LIBELFTC_PRIVATE(msg), sizeof(LIBELFTC_PRIVATE(msg)),
		    "%s", _libelftc_errors[error]);
}

int
elftc_errno(void)
{

	return (LIBELFTC_PRIVATE(error));
}

const char *
elftc_errmsg(void)
{

	if (LIBELFTC_PRIVATE(error) == 0)
		return (NULL);

	return (LIBELFTC_PRIVATE(msg));
}
