/*-
 * Copyright (c) 2009,2010 Kai Wang
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer
 *    in this position and unchanged.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR(S) ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $FreeBSD: users/kaiwang27/elftc/libelftc.h 392 2009-05-31 19:17:46Z kaiwang27 $
 */

#ifndef	_LIBELFTC_H_
#define	_LIBELFTC_H_

#include <libelf.h>

/*
 * Libelftc error numbers.
 */
enum {
	ELFTC_E_NONE,		/* No error. */
	ELFTC_E_ARGUMENT,	/* Invalid argument. */
	ELFTC_E_IO,		/* I/O error. */
	ELFTC_E_LIBARCHIVE,	/* libarchive error. */
	ELFTC_E_LIBELF,		/* libelf error. */
	ELFTC_E_RESOURCE,	/* Resource exhaustion. */
	ELFTC_E_SEQUENCE,	/* API calls out of sequence. */
	ELFTC_E_UNIMPL,		/* Feature is unimplemented. */
	ELFTC_E_NUM,		/* Max error number. */
};

/*
 * Archive API types.
 */

typedef struct _Ar_Obj Ar_Obj;

typedef enum {
	ELFTC_AB_LIBELF,
	ELFTC_AB_LIBELFTC,
	ELFTC_AB_LIBARCHIVE,
	ELFTC_AB_MAX,
} Ar_Backend;

typedef enum {
	ELFTC_AC_NULL,
	ELFTC_AC_READ,
	ELFTC_AC_WRITE,
} Ar_Cmd;

typedef enum {
	ELFTC_ACP_NONE,
	ELFTC_ACP_GZIP,
	ELFTC_ACP_BZIP2,
} Ar_Compress;

typedef enum {
	ELFTC_AF_NONE,
	ELFTC_AF_GNU,
	ELFTC_AF_BSD,
	ELFTC_AF_LLVM,
} Ar_Flavor;

/*
 * BFD Target API types.
 */

typedef struct _Bfd_Target Bfd_Target;

typedef enum {
	ETF_NONE,
	ETF_ELF,
	ETF_BINARY,
	ETF_SREC,
	ETF_IHEX
} Bfd_Target_Flavor;

/*
 * Archive API flags.
 */

#define	ELFTC_AR_NOARSYM	0x00000001U
#define	ELFTC_AR_SEQUENTIAL	0x00000002U

/*
 * Demangler flags.
 */

/* Name mangling style. */
#define	ELFTC_DEM_ARM		0x00000001U
#define	ELFTC_DEM_GNU2		0x00000002U
#define	ELFTC_DEM_GNU3		0x00000004U

/* Demangling behaviour control. */
#define	ELFTC_DEM_NOPARAM	0x00010000U

__BEGIN_DECLS
Ar_Obj			*elftc_ar_open_fd(int, Ar_Cmd, Ar_Backend, int);
Ar_Obj			*elftc_ar_open_file(const char *, Ar_Cmd, Ar_Backend,
			    int);
Ar_Obj			*elftc_ar_open_memory(void *, size_t, Ar_Backend, int);
int			 elftc_ar_set_flavor(Ar_Obj *, Ar_Flavor);
int			 elftc_ar_set_compression(Ar_Obj *, Ar_Compress);
int			 elftc_ar_next(Ar_Obj *);
int			 elftc_ar_rand(Ar_Obj *, off_t);
int			 elftc_ar_rewind(Ar_Obj *);
Elf_Arsym		*elftc_ar_get_arsym(Ar_Obj *, size_t *);
Elf_Arhdr		*elftc_ar_get_arhdr(Ar_Obj *);
void			*elftc_ar_get_data(Ar_Obj *, size_t *);
Elf			*elftc_ar_get_elf(Ar_Obj *);
int			 elftc_ar_extract(Ar_Obj *, int flags);
int			 elftc_ar_new_arhdr(Ar_Obj *, Elf_Arhdr *);
int			 elftc_ar_new_data(Ar_Obj *, void *, size_t);
int			 elftc_ar_close(Ar_Obj *);
Bfd_Target		*elftc_bfd_find_target(const char *tgt_name);
Bfd_Target_Flavor	 elftc_bfd_target_flavor(Bfd_Target *tgt);
unsigned int		 elftc_bfd_target_byteorder(Bfd_Target *tgt);
unsigned int		 elftc_bfd_target_class(Bfd_Target *tgt);
int		 	 elftc_demangle(const char *mangledname, char *buffer,
			     size_t bufsize, unsigned int flags);
const char		*elftc_errmsg(void);
int			elftc_errno(void);
__END_DECLS

#endif	/* _LIBELFTC_H_ */
