/*-
 * Copyright (c) 2010 Kai Wang
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <assert.h>
#include <libelftc.h>

#include "_libelftc.h"

Ar_Obj *
_elftc_ar_alloc_object(Ar_Cmd cmd, Ar_Backend ab, int flags)
{
	Ar_Obj *ao;

	/* Initialise libelf library. */
	if (elf_version(EV_CURRENT) == EV_NONE) {
		ELFTC_SET_ELF_ERROR("elf_version");
		return (NULL);
	}

	if ((ao = calloc(1, sizeof(*ao))) == NULL) {
		ELFTC_SET_ERROR(RESOURCE, 0);
		return (NULL);
	}

	ao->ao_cmd = cmd;
	ao->ao_be = ab;
	ao->ao_flags = flags;

	TAILQ_INIT(&ao->ao_amlist);

	return (ao);
}

void
_elftc_ar_dealloc_object(Ar_Obj *ao)
{

	/* TODO Free Ar_Obj fields here. */

	free(ao);
}

